//
// $Id: wset.h,v 1.1 2002/05/23 22:18:52 carzanig Exp $
//
#ifndef WSET_H
#define WSET_H

#include <stdlib.h>
#include <map>
#include <vector>
#include <list>

using namespace std;

typedef unsigned int weight_t;

typedef list<string> filter_container;
typedef vector<filter_container> if_container;

/** "weighted" set
 *
 *  represents a set of elements, each having a "weight" parameter
 **/
template <class T>
class wset {
private:
    map<weight_t, T>	wmap;
    weight_t		weight_size;
    static weight_t rand_normal(weight_t max) {
	return (weight_t) (1.0*max*rand()/(RAND_MAX+1.0));
    }

 public:
    void clear() {
	wmap.clear();
	weight_size = 0;
    }

    void add(weight_t w, const T & v) {
	if (w > 0) {
	    weight_size += w;
	    wmap.insert(wmap.end(), 
			map<weight_t, T>::value_type(weight_size - 1, v));
	}
    }

    T rand_pick() const {
	return (*wmap.lower_bound(rand_normal(weight_size))).second;
    }

    bool read_from(istream & in) {
	clear();
	weight_t w;
	T v;
	while (in >> w >> v) 
	    add(w, v);
	return true;
    }
};
#endif
